CON
  '' I/O SETTINGS

   _clkmode  = xtal1 + pll16x
   _xinfreq  = 5_000_000

  ''For searching strings
  HEAP_SIZE     = 2000
  STR_MAX_LENGTH  = 64  
  
  '' Adjust for typical location of the keyboard. Usually 26 on the demoboard
  
  keyboard  = 26
  video     = 12

  '' SD Card pins
  spiDO     = 3
  spiClk    = 1
  spiDI     = 2
  spiCS     = 0

  '' Basic Color Settings

  BLACK             = $02 
  GREY              = $03
  LWHITE            = $06
  BWHITE            = $07
  LMAGENTA          = $A3 
  WHITE             = $AF
  RED               = $A2
  LRED              = $1A
  LPURPLE           = $1B
  PURPLE            = $03 
  LGREEN            = $1C
  SKYBLUE           = $1D
  YELLOW            = $1E
  BYELLOW           = $A6
  LGREY             = $1F
  DGREY             = $08
  GREEN             = $A4
  ORANGE            = $06
  BLUE              = $A1 
  CYAN              = $1C
  MAGENTA           = $FC
  LYELLOW           = $1E

  CR                            = $0D
  LF                            = $0A
  TAB                           = $09
  BKS                           = $08

  ''Keycodes
  UP    = 194
  DOWN  = 195
  LEFT  = 192
  RIGHT = 193
  ENTER = 13
  SPACE = 32

OBJ

  key   : "comboKeyboard"             ' Keyboard Driver
  text  : "aigeneric_driver_mod"          ' AIGeneric Video Driver
  fsrw  : "fsrwFemto"                 ' SD Routines
  pause : "Pause"
  strg  : "strings" 

VAR

  'For string lib
  byte  Hp
  long Heap_array[HEAP_SIZE]
  byte ostr[STR_MAX_LENGTH]

  'SD card vars
  long ioControl[2]
  byte tbuf[20]

  'Program vars
  byte command[32]
  byte strbuff[5]       
  byte page[960]
  long scroll
  byte scrollColor

  long results[25]
  
PUB mainProgram  | i

  Init
  chooseFile

PUB Init
  'Start the keyboard, video, sd card, pause, and string modules
  key.start(keyboard)
  text.start(video)
  fsrw.start(@ioControl)
  pause.start
  Hp := strg.start(@heap_array, HEAP_SIZE)  

  'initialize some vars
  scroll := 0
  scrollColor := WHITE
  
  '' Check for required SD interface and PS2 Keyboard
  if \fsrw.mount(spiDO,spiClk,spiDI,spiCS) < 0
     text.str(string(13,"No SD card present.",CR))
     text.str(string("SD card expected on: "))
     text.dec(spiDO)
     text.str(string(", "))
     text.dec(spiClk)
     text.str(string(", "))
     text.dec(spiDI)
     text.str(string(", "))
     text.dec(spiCS)
     text.str(string(13,CR,"System halted.",CR)) 
     abort 
  waitcnt(clkfreq + cnt)
  if not key.present                                
     text.str(string(13,"No keyboard present",CR))
     text.str(string("Keyboard expected on: "))
     text.dec(keyboard)
     text.str(string(13,CR,"System halted.",CR))
     abort

PUB chooseFile

  text.cls
  text.str(string("Enter search term:", CR))
  input(@command, 13)
  search(@command)

PUB loadFile(name) | i, keyval, read, inGarb, count
  'Fill page buffer with end chars
  repeat i from 0 to 959
    page[i] := 0

  'Used to get rid of unwanted html formatting  
  inGarb := false

  'Error check
  read := fsrw.popen(name,"r")
  if read<0
    text.str(string("File did not exist.", CR))

    pause.pause(1000)

    chooseFile

  'Read from the file up to the scroll offset  
  read := 10
  count := 0
  repeat until count>scroll
    read := fsrw.pgetc

    if read == 60
      inGarb := true
     
    if inGarb == false
      if not read==13
        count++  
     
    if read == 62
      inGarb := false  

  'Read the file and fill the page buffer
  count := 0
  repeat until read < 0 or count>959
    read := fsrw.pgetc

    if read == 60
      inGarb := true
     
    if inGarb == false
      if not read==13
        page[count] := read
        count++  
     
    if read == 62
      inGarb := false
               
  fsrw.pclose
  text.cls
  dispFile

PUB dispFile | i, keyval

  'The contrast helps with reading
  if scrollColor==WHITE
    scrollColor:=YELLOW
  else
    scrollColor:=WHITE

  text.color(scrollColor)

  'Display the file
  repeat i from 0 to 959
    if not page[i]==0
      if not page[i]==10
        text.out(page[i])

  'Check for user input
  repeat
    keyval := key.key

    if keyval==203
      scroll := 0
      chooseFile

    if keyval==UP and scroll>0
      scroll -= 960
      loadFile(@tbuf)
        
    if keyval==DOWN
      scroll += 960
      loadFile(@tbuf) 
    
PUB search(term) | rCount, fCount

  text.cls
  rCount := -1
  fCount := 0 
  fsrw.opendir
  repeat while fsrw.nextfile(@tbuf) == 0
    if not strg.find_next(@tbuf, 0, term) == -1
      rCount++
      if rCount<25
        results[rCount] := fCount
      else
        text.cls
        text.str(string("Result array full."))
        pause.pause(3000)
        list(rCount)

    fCount++
    text.str(tbuf)

  if rCount==-1
    text.str(string(CR, "The file was not found."))
    pause.pause(1000)
    chooseFile
  else
    list(rCount)

PUB list(rNum) | keyval, selection, rCount, fCount

  text.cls
  rCount := 0
  fCount := 0 
  fsrw.opendir
  repeat while fsrw.nextfile(@tbuf) == 0
    if fCount==results[rCount]
      text.out(" ")
      text.str(@tbuf)
      text.str(string(CR))
      rCount++

    fCount++

  selection := 0
  text.poke(0, selection, WHITE, $1F)
  repeat
    keyval := key.key

    if keyval==203
      chooseFile

    if keyval==UP
      if selection>0
        text.poke(0, selection, WHITE, " ")
        selection--
        text.poke(0, selection, WHITE, $1F)
    if keyval==DOWN
      if selection<rNum
        text.poke(0, selection, WHITE, " ")
        selection++
        text.poke(0, selection, WHITE, $1F)

    if keyval==ENTER
      rCount := 0
      fCount := 0 
      fsrw.opendir
      repeat while fsrw.nextfile(@tbuf) == 0
        if fCount==results[rCount]
          if rCount==selection
            loadFile(@tbuf)
          rCount++
       
        fCount++

'Taken from PropDOS    
PUB input(data, size) | i, keystroke, len

  repeat i from 0 to 32
    command[i] := 0

  keystroke:=0
  len:=0
  text.out(02)
  repeat' until keystroke == 13 'Exit when Carriage Return is pressed
    keystroke := key.key
   
     if keystroke == 208
        keystroke:=36
    
     if keystroke == $C8       ' Check for Backspace Key
       if len > 0
        byte[data--] := 0
        size++
        len--
        text.out(8)
        text.out(32) 'These lines erase the cursor on backspace
        text.out(8)
        
        text.out(8)
        text.out(32)
        text.out(8)
        text.out(02)
        keystroke := 0
   
     if keystroke == $C8        'Clean up backspace if all is cleared
        keystroke:= 0
   
     if keystroke == 13
        text.out(8)
        text.out(32)
        text.out(8)
        text.out(13)
        quit
        
     if keystroke > 10 
       if size > 1
         text.out(8)
         text.out(keystroke)
         text.out(02)
         
         ''text.dec(keystroke) 'for troubleshooting
         if keystroke <> 13 
           byte[data++] := keystroke
           size--
           len++

'Taken off the forums, not my code
PUB strPos (strAddr, searchAddr, offset) | searchsize
'' returns location of first occurrence of search in str, returns -1 if search is not found 
  searchsize := strsize(searchAddr)

  REPEAT UNTIL (offset + searchsize > STR_MAX_LENGTH)
    IF (strcomp(substr(strAddr, offset++, searchsize), searchAddr))
      RETURN offset - 1
  RETURN -1

PUB substr (strAddr, start, count) | i
'' returns part of a string for count bytes starting from start byte
  bytefill(@ostr, 0, STR_MAX_LENGTH)
  i~

  bytemove(@ostr, strAddr + start, count) 
  return @ostr  